function [alph, bet, ld, varargout] = learn_lda(data, K, param_folder, varargin)
% [alph, bet] = learn_lda(data,K,param_folder)
% This function takes a data set and runs lda on it
% data - data matrix of counts, rows=words, columns=documents
% K - number of clusters(topics)
% param_folder - folder in whcih to store the variational parameters
%                and current progress
% tol - stopping tolerance for log-likelihood (default=1e-5)

if(nargin == 3)
    num_iter = 100;
    tol = 1e-4;
elseif(nargin == 4)
    opt_param = varargin{1};
    if(isfield(opt_param,'num_iter'))
        num_iter = opt_param.num_iter;
    end
    if(isfield(opt_param,'seed'))
        rand('state',opt_param.seed);
    end   
    if(isfield(opt_param,'tol'))
        tol = opt_param.tol;
    end   
else
    error('Not Correct Number of input parameters')
end

state = rand('state');
[V,D] = size(data);

% Initialize prior parameter
alph = 1e-3.*rand(K,1);

% Initialize variational parameters
for d = 1:D
  index = find(data(:,d)>0);
  
  phi = normalize(repmat(1/K,length(index),K)+rand(length(index),K));
  gam = alph + repmat(sum(data(:,d))./K,K,1);
  ll = inf;
  
  save([param_folder,'/var',num2str(d)],'phi','gam','ll'); 
end

h = waitbar(0,'Running EM algorithm...');
% Start the EM variational
t=0;
ld_diff = inf;
while(t < num_iter && ld_diff > tol)
    t=t+1;
     waitbar(t/num_iter,h,['Iteration ' num2str(t) ' of ' ...
            num2str(num_iter)]) 
    
    % M-step
    %disp(['Iteration ' num2str(t) ': M-step']);
    [alph,bet] = lda_m_step(data,K,param_folder,0);
    
    % E-step
    %disp(['Iteration ' num2str(t) ': E-step']);
    for d = 1:D
        %disp(['Document: ' num2str(d) ' of ' num2str(D)]);
        [phi, gam, ll(d)] = lda_e_step(data(:,d), alph, bet);
        save([param_folder,'/var',num2str(d)],'phi','gam', 'll'); 
    end 
    
    ld(t) = sum(ll);
    if(t>1)
        ld_diff = (ld(t)-ld(t-1))/abs(ld(t-1));
        if(ld_diff < 0) 
            disp(ld_diff)
            warning('Decreased log-likelihood');
        end
    else
        ld_diff = Inf;
    end
    
    disp(['Log-likelihood change: ',num2str(ld_diff*100,'%4.2d'),'%'])
    
    % After one iteration of learning - save the current state
    save([param_folder,'/current'],'alph','bet','ld','ld_diff');
end
close(h)

if(nargout == 4)
    varargout(1) = {state};
end